@@ -22,15 +22,15 @@ When subsequent requests for the same image file are received,
 the local cached copy of the image file is returned.
 """
 
+import http.client as http
 import re
-import six
 
 from oslo_log import log as logging
-from six.moves import http_client as http
 import webob
 
 from glance.api.common import size_checked_iter
 from glance.api import policy
+from glance.api.v2 import policy as api_policy
 from glance.common import exception
 from glance.common import utils
 from glance.common import wsgi
@@ -38,7 +38,6 @@ import glance.db
 from glance.i18n import _LE, _LI
 from glance import image_cache
 from glance import notifier
-import glance.registry.client.v1.api as registry
 
 LOG = logging.getLogger(__name__)
 
@@ -96,28 +95,11 @@ class CacheFilter(wsgi.Middleware):
             if image_id != 'detail':
                 return (version, method, image_id)
 
-    def _enforce(self, req, action, target=None):
+    def _enforce(self, req, image):
         """Authorize an action against our policies"""
-        if target is None:
-            target = {}
-        try:
-            self.policy.enforce(req.context, action, target)
-        except exception.Forbidden as e:
-            LOG.debug("User not permitted to perform '%s' action", action)
-            raise webob.exc.HTTPForbidden(explanation=e.msg, request=req)
-
-    def _get_v1_image_metadata(self, request, image_id):
-        """
-        Retrieves image metadata using registry for v1 api and creates
-        dictionary-like mash-up of image core and custom properties.
-        """
-        try:
-            image_metadata = registry.get_image_metadata(request.context,
-                                                         image_id)
-            return utils.create_mashup_dict(image_metadata)
-        except exception.NotFound as e:
-            LOG.debug("No metadata found for image '%s'", image_id)
-            raise webob.exc.HTTPNotFound(explanation=e.msg, request=request)
+        api_pol = api_policy.ImageAPIPolicy(req.context, image,
+                                            self.policy)
+        api_pol.download_image()
 
     def _get_v2_image_metadata(self, request, image_id):
         """
@@ -132,7 +114,7 @@ class CacheFilter(wsgi.Middleware):
             # _process_v2_request call.
             request.environ['api.cache.image'] = image
 
-            return policy.ImageTarget(image)
+            return (image, policy.ImageTarget(image))
         except exception.NotFound as e:
             raise webob.exc.HTTPNotFound(explanation=e.msg, request=request)
 
@@ -164,26 +146,26 @@ class CacheFilter(wsgi.Middleware):
             return None
 
         method = getattr(self, '_get_%s_image_metadata' % version)
-        image_metadata = method(request, image_id)
+        image, metadata = method(request, image_id)
 
         # Deactivated images shall not be served from cache
-        if image_metadata['status'] == 'deactivated':
+        if metadata['status'] == 'deactivated':
             return None
 
-        try:
-            self._enforce(request, 'download_image', target=image_metadata)
-        except exception.Forbidden:
-            return None
+        # NOTE(abhishekk): This means image is present in cache and before
+        # request is coming to API we are enforcing this check in the
+        # middleware
+        self._enforce(request, image)
 
         LOG.debug("Cache hit for image '%s'", image_id)
         image_iterator = self.get_from_cache(image_id)
         method = getattr(self, '_process_%s_request' % version)
 
         try:
-            return method(request, image_id, image_iterator, image_metadata)
+            return method(request, image_id, image_iterator, metadata)
         except exception.ImageNotFound:
             msg = _LE("Image cache contained image file for image '%s', "
-                      "however the registry did not contain metadata for "
+                      "however the database did not contain metadata for "
                       "that image!") % image_id
             LOG.error(msg)
             self.cache.delete_cached_image(image_id)
@@ -237,8 +219,7 @@ class CacheFilter(wsgi.Middleware):
         # https://github.com/Pylons/webob/issues/86
         response.headers['Content-Type'] = 'application/octet-stream'
         if image.checksum:
-            response.headers['Content-MD5'] = (image.checksum.encode('utf-8')
-                                               if six.PY2 else image.checksum)
+            response.headers['Content-MD5'] = image.checksum
         response.headers['Content-Length'] = str(image.size)
         return response
 
@@ -294,16 +275,20 @@ class CacheFilter(wsgi.Middleware):
             LOG.error(_LE("Checksum header is missing."))
 
         # fetch image_meta on the basis of version
-        image_metadata = None
+        image = None
         if version:
             method = getattr(self, '_get_%s_image_metadata' % version)
-            image_metadata = method(resp.request, image_id)
+            image, metadata = method(resp.request, image_id)
         # NOTE(zhiyan): image_cache return a generator object and set to
         # response.app_iter, it will be called by eventlet.wsgi later.
         # So we need enforce policy firstly but do it by application
         # since eventlet.wsgi could not catch webob.exc.HTTPForbidden and
         # return 403 error to client then.
-        self._enforce(resp.request, 'download_image', target=image_metadata)
+        # FIXME(abhishekk): This policy check here is not necessary as this
+        # will hit only during first image download i.e. while image is not
+        # present in cache. We already enforced same check in API layer and
+        # enforcing same check here again makes no sense.
+        self._enforce(resp.request, image)
 
         resp.app_iter = self.cache.get_caching_iter(image_id, image_checksum,
                                                     resp.app_iter)
