@@ -20,7 +20,6 @@ from glance_store import backend
 from glance_store import location
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 import webob.exc
 
@@ -67,7 +66,7 @@ class ImageDataController(object):
         except Exception as e:
             msg = (_LE("Unable to restore image %(image_id)s: %(e)s") %
                    {'image_id': image.image_id,
-                    'e': encodeutils.exception_to_unicode(e)})
+                    'e': e})
             LOG.exception(msg)
 
     def _unstage(self, image_repo, image, staging_store):
@@ -154,7 +153,7 @@ class ImageDataController(object):
                 except Exception as e:
                     LOG.info(_LI("Unable to create trust: %s "
                                  "Use the existing user token."),
-                             encodeutils.exception_to_unicode(e))
+                             e)
 
                 image_repo.save(image, from_state='queued')
                 ks_quota.enforce_image_count_uploading(req.context,
@@ -176,9 +175,9 @@ class ImageDataController(object):
                     if refresher is not None:
                         refresher.release_resources()
                 except Exception as e:
-                    LOG.info(_LI("Unable to delete trust %(trust)s: %(msg)s"),
+                    LOG.info(_LI("Unable to delete trust %(trust)s: %(err)s"),
                              {"trust": refresher.trust_id,
-                              "msg": encodeutils.exception_to_unicode(e)})
+                              "err": e})
 
             except (glance_store.NotFound,
                     exception.ImageNotFound,
@@ -213,10 +212,10 @@ class ImageDataController(object):
         except ValueError as e:
             LOG.debug("Cannot save data for image %(id)s: %(e)s",
                       {'id': image_id,
-                       'e': encodeutils.exception_to_unicode(e)})
+                       'e': e})
             self._restore(image_repo, image)
             raise webob.exc.HTTPBadRequest(
-                explanation=encodeutils.exception_to_unicode(e))
+                explanation=str(e))
 
         except glance_store.StoreAddDisabled:
             msg = _("Error in store configuration. Adding images to store "
@@ -227,8 +226,7 @@ class ImageDataController(object):
                                      content_type='text/plain')
 
         except exception.InvalidImageStatusTransition as e:
-            msg = encodeutils.exception_to_unicode(e)
-            LOG.exception(msg)
+            LOG.exception(str(e))
             raise webob.exc.HTTPConflict(explanation=e.msg, request=req)
 
         except exception.Forbidden:
@@ -241,24 +239,21 @@ class ImageDataController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
 
         except glance_store.StorageFull as e:
-            msg = _("Image storage media "
-                    "is full: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("Image storage media is full: %s") % e
             LOG.error(msg)
             self._restore(image_repo, image)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
         except exception.StorageQuotaFull as e:
-            msg = _("Image exceeds the storage "
-                    "quota: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("Image exceeds the storage quota: %s") % e
             LOG.error(msg)
             self._restore(image_repo, image)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
         except exception.ImageSizeLimitExceeded as e:
-            msg = _("The incoming image is "
-                    "too large: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("The incoming image is too large: %s") % e
             LOG.error(msg)
             self._restore(image_repo, image)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
@@ -277,17 +272,22 @@ class ImageDataController(object):
                                                      request=req)
 
         except glance_store.StorageWriteDenied as e:
-            msg = _("Insufficient permissions on image "
-                    "storage media: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("Insufficient permissions on image storage media: %s") % e
             LOG.error(msg)
             self._restore(image_repo, image)
             raise webob.exc.HTTPServiceUnavailable(explanation=msg,
                                                    request=req)
 
+        except glance_store.Invalid as e:
+            LOG.error(e.message)
+            if image.status not in ('queued', 'deleted'):
+                self._restore(image_repo, image)
+            raise webob.exc.HTTPBadRequest(
+                explanation=str(e))
+
         except cursive_exception.SignatureVerificationError as e:
             msg = (_LE("Signature verification failed for image %(id)s: %(e)s")
-                   % {'id': image_id,
-                      'e': encodeutils.exception_to_unicode(e)})
+                   % {'id': image_id, 'e': e})
             LOG.error(msg)
             self._restore(image_repo, image)
             raise webob.exc.HTTPBadRequest(explanation=msg)
@@ -309,6 +309,8 @@ class ImageDataController(object):
 
     @utils.mutating
     def stage(self, req, image_id, data, size):
+        if size is None:
+            size = 0
         try:
             ks_quota.enforce_image_staging_total(req.context,
                                                  req.context.owner)
@@ -375,10 +377,11 @@ class ImageDataController(object):
             ks_quota.enforce_image_count_uploading(req.context,
                                                    req.context.owner)
             try:
-                uri, size, id, store_info = staging_store.add(
+                uri, image_size, id, store_info = staging_store.add(
                     image_id, utils.LimitingReader(
-                        utils.CooperativeReader(data), CONF.image_size_cap), 0)
-                image.size = size
+                        utils.CooperativeReader(data), CONF.image_size_cap),
+                    size)
+                image.size = image_size
             except glance_store.Duplicate:
                 msg = _("The image %s has data on staging") % image_id
                 raise webob.exc.HTTPConflict(explanation=msg)
@@ -395,24 +398,28 @@ class ImageDataController(object):
             raise webob.exc.HTTPNotFound(explanation=e.msg)
 
         except glance_store.StorageFull as e:
-            msg = _("Image storage media "
-                    "is full: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("Image storage media is full: %s") % e
             LOG.error(msg)
             self._unstage(image_repo, image, staging_store)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
+        except glance_store.Invalid as e:
+            LOG.error(e.message)
+            if image.status not in ('queued', 'deleted'):
+                self._restore(image_repo, image)
+            raise webob.exc.HTTPBadRequest(
+                explanation=str(e))
+
         except exception.StorageQuotaFull as e:
-            msg = _("Image exceeds the storage "
-                    "quota: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("Image exceeds the storage quota: %s") % e
             LOG.debug(msg)
             self._unstage(image_repo, image, staging_store)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
                                                       request=req)
 
         except exception.ImageSizeLimitExceeded as e:
-            msg = _("The incoming image is "
-                    "too large: %s") % encodeutils.exception_to_unicode(e)
+            msg = _("The incoming image is too large: %s") % e
             LOG.debug(msg)
             self._unstage(image_repo, image, staging_store)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg,
@@ -426,15 +433,14 @@ class ImageDataController(object):
 
         except glance_store.StorageWriteDenied as e:
             msg = _("Insufficient permissions on image "
-                    "storage media: %s") % encodeutils.exception_to_unicode(e)
+                    "storage media: %s") % e
             LOG.error(msg)
             self._unstage(image_repo, image, staging_store)
             raise webob.exc.HTTPServiceUnavailable(explanation=msg,
                                                    request=req)
 
         except exception.InvalidImageStatusTransition as e:
-            msg = encodeutils.exception_to_unicode(e)
-            LOG.debug(msg)
+            LOG.debug(str(e))
             raise webob.exc.HTTPConflict(explanation=e.msg, request=req)
 
         except Exception:
@@ -467,6 +473,20 @@ class ImageDataController(object):
 
 class RequestDeserializer(wsgi.JSONRequestDeserializer):
 
+    def _get_image_size(self, request):
+        try:
+            size = request.headers.get('x-openstack-image-size')
+            if size is not None:
+                image_size = int(size)
+            else:
+                # If header is missing, fall back to content_length or None
+                image_size = request.content_length or None
+        except (ValueError, TypeError):
+            # Raised if conversion to int fails or value is None
+            raise webob.exc.HTTPBadRequest(explanation=_(
+                "Invalid or missing image size in request headers."))
+        return image_size
+
     def upload(self, request):
         try:
             request.get_content_type(('application/octet-stream',))
@@ -476,8 +496,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if self.is_valid_encoding(request) and self.is_valid_method(request):
             request.is_body_readable = True
 
-        image_size = request.content_length or None
-        return {'size': image_size, 'data': request.body_file}
+        return {'size': self._get_image_size(request),
+                'data': request.body_file}
 
     def stage(self, request):
         if "glance-direct" not in CONF.enabled_import_methods:
@@ -491,8 +511,8 @@ class RequestDeserializer(wsgi.JSONRequestDeserializer):
         if self.is_valid_encoding(request) and self.is_valid_method(request):
             request.is_body_readable = True
 
-        image_size = request.content_length or None
-        return {'size': image_size, 'data': request.body_file}
+        return {'size': self._get_image_size(request),
+                'data': request.body_file}
 
 
 class ResponseSerializer(wsgi.JSONResponseSerializer):
