@@ -16,7 +16,6 @@ import http.client as http
 
 import glance_store
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 import webob.exc
 
 from glance.api import policy
@@ -62,14 +61,13 @@ class Controller(object):
             LOG.warning(msg)
             raise webob.exc.HTTPForbidden(explanation=msg)
         except exception.Invalid as e:
-            msg = (_("Could not update image: %s")
-                   % encodeutils.exception_to_unicode(e))
+            msg = (_("Could not update image: %s") % e)
             LOG.warning(msg)
             raise webob.exc.HTTPBadRequest(explanation=msg)
         except exception.ImageTagLimitExceeded as e:
             msg = (_("Image tag limit exceeded for image %(id)s: %(e)s:")
                    % {"id": image_id,
-                      "e": encodeutils.exception_to_unicode(e)})
+                      "e": e})
             LOG.warning(msg)
             raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
 
