@@ -23,7 +23,6 @@ import glance_store
 from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_serialization.jsonutils as json
-from oslo_utils import encodeutils
 from oslo_utils import uuidutils
 import webob.exc
 
@@ -70,12 +69,9 @@ class TasksController(object):
         # NOTE(rosmaita): access to this call is enforced in the deserializer
 
         ctxt = req.context
-        task_factory = self.gateway.get_task_factory(
-            ctxt, authorization_layer=False)
-        executor_factory = self.gateway.get_task_executor_factory(
-            ctxt, authorization_layer=False)
-        task_repo = self.gateway.get_task_repo(ctxt,
-                                               authorization_layer=False)
+        task_factory = self.gateway.get_task_factory(ctxt)
+        executor_factory = self.gateway.get_task_executor_factory(ctxt)
+        task_repo = self.gateway.get_task_repo(ctxt)
         try:
             new_task = task_factory.new_task(
                 task_type=task['type'],
@@ -90,7 +86,7 @@ class TasksController(object):
             pool.spawn(new_task.run, task_executor)
         except exception.Forbidden as e:
             msg = (_LW("Forbidden to create task. Reason: %(reason)s")
-                   % {'reason': encodeutils.exception_to_unicode(e)})
+                   % {'reason': e})
             LOG.warning(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         return new_task
@@ -109,8 +105,7 @@ class TasksController(object):
             limit = CONF.limit_param_default
         limit = min(CONF.api_limit_max, limit)
 
-        task_repo = self.gateway.get_task_stub_repo(
-            req.context, authorization_layer=False)
+        task_repo = self.gateway.get_task_stub_repo(req.context)
         try:
             tasks = task_repo.list(marker, limit, sort_key,
                                    sort_dir, filters)
@@ -118,10 +113,10 @@ class TasksController(object):
                 result['next_marker'] = tasks[-1].task_id
         except (exception.NotFound, exception.InvalidSortKey,
                 exception.InvalidFilterRangeValue) as e:
-            LOG.warning(encodeutils.exception_to_unicode(e))
+            LOG.warning(str(e))
             raise webob.exc.HTTPBadRequest(explanation=e.msg)
         except exception.Forbidden as e:
-            LOG.warning(encodeutils.exception_to_unicode(e))
+            LOG.warning(str(e))
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         result['tasks'] = tasks
         return result
@@ -130,20 +125,19 @@ class TasksController(object):
     def get(self, req, task_id):
         _enforce_access_policy(self.policy, req)
         try:
-            task_repo = self.gateway.get_task_repo(
-                req.context, authorization_layer=False)
+            task_repo = self.gateway.get_task_repo(req.context)
             task = task_repo.get(task_id)
         except exception.NotFound as e:
             msg = (_LW("Failed to find task %(task_id)s. Reason: %(reason)s")
                    % {'task_id': task_id,
-                      'reason': encodeutils.exception_to_unicode(e)})
+                      'reason': e})
             LOG.warning(msg)
             raise webob.exc.HTTPNotFound(explanation=e.msg)
         except exception.Forbidden as e:
             msg = (_LW("Forbidden to get task %(task_id)s. Reason:"
                        " %(reason)s")
                    % {'task_id': task_id,
-                      'reason': encodeutils.exception_to_unicode(e)})
+                      'reason': e})
             LOG.warning(msg)
             raise webob.exc.HTTPForbidden(explanation=e.msg)
         return task
@@ -356,7 +350,8 @@ _TASK_SCHEMA = {
         "description": _("The type of task represented by this content"),
         "enum": [
             "import",
-            "api_image_import"
+            "api_image_import",
+            "location_import",
         ],
         "type": "string"
     },
