@@ -19,13 +19,12 @@ import os
 from oslo_concurrency import processutils as putils
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
+from oslo_utils.imageutils import format_inspector
 from taskflow.patterns import linear_flow as lf
 from taskflow import task
 
 from glance.async_ import utils
-from glance.common import format_inspector
 from glance.i18n import _, _LI
 
 LOG = logging.getLogger(__name__)
@@ -81,6 +80,37 @@ class _ConvertImage(task.Task):
         with self.action_wrapper as action:
             return self._execute(action, file_path, **kwargs)
 
+    def _inspect_path(self, path):
+        """Return a FormatInspector for path.
+
+        This encapsulates the act of inspecting a file, running safety checks,
+        and raising/logging if anything goes wrong. If nothing fails, return
+        the FileInspector for the file.
+        """
+        # Use our own cautious inspector module (if we have one for this
+        # format) to make sure a file is the format the submitter claimed
+        # it is and that it passes some basic safety checks _before_ we run
+        # qemu-img on it.
+        # See https://bugs.launchpad.net/nova/+bug/2059809 for details.
+        try:
+            inspector = format_inspector.detect_file_format(path)
+            inspector.safety_check()
+        except format_inspector.SafetyCheckFailed as e:
+            nonfatal = set(CONF.image_format.gpt_safety_checks_nonfatal)
+            fatal = e.failures.keys() - nonfatal
+            if inspector.NAME == 'gpt' and not fatal:
+                LOG.warning('Non-fatal %s', e)
+            else:
+                LOG.error('%s %s', str(inspector), e)
+                raise RuntimeError('Image has disallowed configuration')
+        except format_inspector.ImageFormatError as e:
+            LOG.error('Image failed format inspection: %s', e)
+            raise RuntimeError('Image format detection failed')
+        except Exception as e:
+            LOG.exception('Unknown error inspecting image format: %s', e)
+            raise RuntimeError('Unable to inspect image')
+        return inspector
+
     def _execute(self, action, file_path, **kwargs):
 
         target_format = CONF.image_conversion.output_format
@@ -91,29 +121,14 @@ class _ConvertImage(task.Task):
                                              'target': target_format}
         self.dest_path = dest_path
         source_format = action.image_disk_format
+        inspector = self._inspect_path(self.src_path)
 
-        # Use our own cautious inspector module (if we have one for this
-        # format) to make sure a file is the format the submitter claimed
-        # it is and that it passes some basic safety checks _before_ we run
-        # qemu-img on it.
-        # See https://bugs.launchpad.net/nova/+bug/2059809 for details.
-        try:
-            inspector = format_inspector.detect_file_format(self.src_path)
-            if not inspector.safety_check():
-                LOG.error('Image failed %s safety check; aborting conversion',
-                          source_format)
-                raise RuntimeError('Image has disallowed configuration')
-        except RuntimeError:
-            raise
-        except format_inspector.ImageFormatError as e:
-            LOG.error('Image claimed to be %s format failed format '
-                      'inspection: %s', source_format, e)
-            raise RuntimeError('Image format detection failed')
-        except Exception as e:
-            LOG.exception('Unknown error inspecting image format: %s', e)
-            raise RuntimeError('Unable to inspect image')
+        detected_format = str(inspector)
+        if detected_format == 'gpt':
+            # FIXME(danms): We need to consider GPT to be raw for compatibility
+            detected_format = 'raw'
 
-        if str(inspector) == 'iso':
+        if detected_format == 'iso':
             if source_format == 'iso':
                 # NOTE(abhishekk): Excluding conversion and preserving image
                 # disk_format as `iso` only
@@ -126,7 +141,7 @@ class _ConvertImage(task.Task):
             LOG.error('Image claimed to be %s format but format '
                       'inspection found: ISO', source_format)
             raise RuntimeError("Image has disallowed configuration")
-        elif str(inspector) != source_format:
+        elif detected_format != source_format:
             LOG.error('Image claimed to be %s format failed format '
                       'inspection', source_format)
             raise RuntimeError('Image format mismatch')
@@ -141,10 +156,9 @@ class _ConvertImage(task.Task):
                                            log_errors=putils.LOG_ALL_ERRORS,)
         except OSError as exc:
             with excutils.save_and_reraise_exception():
-                exc_message = encodeutils.exception_to_unicode(exc)
                 msg = ("Failed to do introspection as part of image "
                        "conversion for %(iid)s: %(err)s")
-                LOG.error(msg, {'iid': self.image_id, 'err': exc_message})
+                LOG.error(msg, {'iid': self.image_id, 'err': exc})
 
         if stderr:
             raise RuntimeError(stderr)
@@ -202,13 +216,24 @@ class _ConvertImage(task.Task):
                                            log_errors=putils.LOG_ALL_ERRORS)
         except OSError as exc:
             with excutils.save_and_reraise_exception():
-                exc_message = encodeutils.exception_to_unicode(exc)
                 msg = "Failed to do image conversion for %(iid)s: %(err)s"
-                LOG.error(msg, {'iid': self.image_id, 'err': exc_message})
+                LOG.error(msg, {'iid': self.image_id, 'err': exc})
 
         if stderr:
             raise RuntimeError(stderr)
 
+        dest_inspector = self._inspect_path(dest_path)
+        dest_format = str(dest_inspector)
+        if dest_format == 'gpt':
+            # FIXME(danms): We need to consider GPT to be raw for compatibility
+            dest_format = 'raw'
+
+        if target_format != dest_format:
+            # If someone hid one format inside another, we should reject it
+            # as we could be about to embed a vmdk in a 'raw' or similar.
+            LOG.error('Image detected as %s after conversion to %s',
+                      dest_format, target_format)
+            raise RuntimeError('Converted image in unexpected format')
         action.set_image_attribute(disk_format=target_format,
                                    container_format='bare')
         new_size = os.stat(dest_path).st_size
