@@ -22,8 +22,6 @@ import os
 
 from oslo_config import cfg
 from oslo_middleware import cors
-from oslo_policy import opts
-from oslo_policy import policy
 from paste import deploy
 
 from glance.i18n import _
@@ -111,6 +109,33 @@ image_format_opts = [
                        "checked during image conversion (if enabled), and "
                        "limits the types of VMDK images we will convert "
                        "from.")),
+    cfg.BoolOpt('require_image_format_match',
+                default=True,
+                help=_('When enabled, glance will inspect the content of '
+                       'uploads and require that they match the disk_format '
+                       'set on the image. A mismatch will abort the upload '
+                       'with an error. Disabling this may be required in the '
+                       'case of false match (or mismatch) issues, but those '
+                       'are bugs that should be filed and fixed. Enabling '
+                       'this feature improves security and consistency by '
+                       'ensuring that images claiming to be a given format '
+                       'have content matching that format.')),
+    cfg.ListOpt('gpt_safety_checks_nonfatal',
+                default=['mbr'],
+                help=_('List of safety check names to consider non-fatal when '
+                       'validating disk_format=gpt images (or images detected '
+                       'as gpt). Since MBR is very loosely-defined and GPT '
+                       '(which is strictly defined) depends on the MBR '
+                       'structure, it is easy to craft a GPT disk image that '
+                       'works, but is not valid according to the spec. While '
+                       'not recommended, it is possible to disable MBR or '
+                       'GPT-specific safety checks if necessary for '
+                       'compatibility. This can include "mbr" and/or "gptmbr" '
+                       'to avoid failing format validation if those checks '
+                       'fail. Use this with caution as it makes glance relax '
+                       'its definition of what a valid GPT disk image looks '
+                       'like and prevents strict adherence '
+                       'to the GPT spec.')),
 ]
 task_opts = [
     cfg.IntOpt('task_time_to_live',
@@ -173,36 +198,6 @@ Related Options:
 ]
 
 common_opts = [
-    cfg.BoolOpt('allow_additional_image_properties', default=True,
-                deprecated_for_removal=True,
-                deprecated_since="Ussuri",
-                deprecated_reason=_("""
-This option is redundant.  Control custom image property usage via the
-'image_property_quota' configuration option.  This option is scheduled
-to be removed during the Victoria development cycle.
-"""),
-                help=_("""
-Allow users to add additional/custom properties to images.
-
-Glance defines a standard set of properties (in its schema) that
-appear on every image. These properties are also known as
-``base properties``. In addition to these properties, Glance
-allows users to add custom properties to images. These are known
-as ``additional properties``.
-
-By default, this configuration option is set to ``True`` and users
-are allowed to add additional properties. The number of additional
-properties that can be added to an image can be controlled via
-``image_property_quota`` configuration option.
-
-Possible values:
-    * True
-    * False
-
-Related options:
-    * image_property_quota
-
-""")),
     cfg.StrOpt('hashing_algorithm',
                default='sha512',
                help=_("""
@@ -255,12 +250,6 @@ Maximum number of properties allowed on an image.
 This enforces an upper limit on the number of additional properties an image
 can have. Any negative value is interpreted as unlimited.
 
-NOTE: This won't have any impact if additional properties are disabled. Please
-refer to ``allow_additional_image_properties``.
-
-Related options:
-    * ``allow_additional_image_properties``
-
 """)),
     cfg.IntOpt('image_tag_quota', default=128,
                help=_("""
@@ -340,8 +329,8 @@ location is where the image data is stored in backend storage. This
 image location is shown under the image property ``direct_url``.
 
 When multiple image locations exist for an image, the best location
-is displayed based on the location strategy indicated by the
-configuration option ``location_strategy``.
+is displayed based on the store weightage assigned for each store
+indicated by the configuration option ``weight``.
 
 NOTES:
     * Revealing image locations can present a GRAVE SECURITY RISK as
@@ -358,7 +347,7 @@ Possible values:
 
 Related options:
     * show_multiple_locations
-    * location_strategy
+    * weight
 
 """)),
     # NOTE(flaper87): The policy.yaml file should be updated and the location
@@ -390,9 +379,9 @@ Show all image locations when returning an image.
 This configuration option indicates whether to show all the image
 locations when returning image details to the user. When multiple
 image locations exist for an image, the locations are ordered based
-on the location strategy indicated by the configuration opt
-``location_strategy``. The image locations are shown under the
-image property ``locations``.
+on the store weightage assigned for each store indicated by the
+configuration option ``weight``. The image locations are shown
+under the image property ``locations``.
 
 NOTES:
     * Revealing image locations can present a GRAVE SECURITY RISK as
@@ -411,8 +400,24 @@ Possible values:
 
 Related options:
     * show_image_direct_url
-    * location_strategy
+    * weight
 
+""")),
+    cfg.BoolOpt('do_secure_hash', default=True,
+                help=_("""
+Calculate hash and checksum for the image.
+
+This configuration option indicates that /v2/images/{image_id}/locations
+POST API will calculate hash and checksum of the image on the fly.
+If False it will silently ignore the hash and checksum calculation.
+
+Possible values:
+    * True
+    * False
+""")),
+    cfg.IntOpt('http_retries', default=3,
+               help=_("""
+The number of times to retry when any operation fails.
 """)),
     cfg.IntOpt('image_size_cap', default=1099511627776, min=1,
                max=9223372036854775808,
@@ -511,6 +516,16 @@ Related options:
 
 """)),
     cfg.StrOpt('metadata_encryption_key',
+               deprecated_for_removal=True,
+               deprecated_since="Dalmatian",
+               deprecated_reason=_("""
+This option doesnt serves the purpose of encryption of location metadata,
+whereas it encrypts location url only for specific APIs. Also if enabled
+this during an upgrade may disrupt existing deployments, as it does not
+support/provide db upgrade script to encrypt existing location URLs.
+Moreover, its functionality for encrypting location URLs is inconsistent
+which is resulting in download failures.
+"""),
                secret=True,
                help=_("""
 AES key for encrypting store location metadata.
@@ -530,6 +545,11 @@ Related options:
 """)),
     cfg.StrOpt('digest_algorithm',
                default='sha256',
+               deprecated_for_removal=True,
+               deprecated_since="Dalmatian",
+               deprecated_reason=_("""
+This option has had no effect since the removal of native SSL support.
+"""),
                help=_("""
 Digest algorithm to use for digital signature.
 
@@ -645,7 +665,6 @@ CONF.register_opts(image_format_opts, group='image_format')
 CONF.register_opts(task_opts, group='task')
 CONF.register_opts(common_opts)
 CONF.register_opts(wsgi_opts, group='wsgi')
-policy.Enforcer(CONF)
 
 
 def parse_args(args=None, usage=None, default_config_files=None):
@@ -752,12 +771,6 @@ def set_config_defaults():
     """This method updates all configuration default values."""
     set_cors_middleware_defaults()
 
-    # TODO(gmann): Remove setting the default value of config policy_file
-    # once oslo_policy change the default value to 'policy.yaml'.
-    # https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
-    DEFAULT_POLICY_FILE = 'policy.yaml'
-    opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
-
 
 def set_cors_middleware_defaults():
     """Update default configuration options for oslo.middleware."""
