@@ -13,10 +13,33 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+
+from keystoneauth1 import session
+from keystoneauth1 import token_endpoint
+from oslo_config import cfg
 from oslo_context import context
 
 from glance.api import policy
 
+CONF = cfg.CONF
+
+
+def get_ksa_client(context):
+    """Returns a keystoneauth Adapter using token from context.
+
+    This will return a simple keystoneauth adapter that can be used to
+    make requests against a remote service using the token provided
+    (and already authenticated) from the user and stored in a
+    RequestContext.
+
+    :param context: User request context
+    :returns: keystoneauth1 Adapter object
+    """
+    auth = token_endpoint.Token(CONF.keystone_authtoken.identity_uri,
+                                context.auth_token)
+    return session.Session(auth=auth)
+
 
 class RequestContext(context.RequestContext):
     """Stores information about the security context.
@@ -26,12 +49,17 @@ class RequestContext(context.RequestContext):
 
     """
 
-    def __init__(self, roles=None,
-                 owner_is_tenant=True, service_catalog=None,
-                 policy_enforcer=None, **kwargs):
+    def __init__(self, service_catalog=None, policy_enforcer=None, **kwargs):
+        # TODO(mriedem): Remove usage of user and tenant from old tests.
+        if 'tenant' in kwargs:
+            # Prefer project_id if passed, otherwise alias tenant as project_id
+            tenant = kwargs.pop('tenant')
+            kwargs['project_id'] = kwargs.get('project_id', tenant)
+        if 'user' in kwargs:
+            # Prefer user_id if passed, otherwise alias user as user_id
+            user = kwargs.pop('user')
+            kwargs['user_id'] = kwargs.get('user_id', user)
         super(RequestContext, self).__init__(**kwargs)
-        self.roles = roles or []
-        self.owner_is_tenant = owner_is_tenant
         self.service_catalog = service_catalog
         self.policy_enforcer = policy_enforcer or policy.Enforcer()
         if not self.is_admin:
@@ -52,18 +80,30 @@ class RequestContext(context.RequestContext):
     @property
     def owner(self):
         """Return the owner to correlate with an image."""
-        return self.tenant if self.owner_is_tenant else self.user
+        return self.project_id
 
     @property
     def can_see_deleted(self):
         """Admins can see deleted by default"""
         return self.show_deleted or self.is_admin
 
+    def elevated(self):
+        """Return a copy of this context with admin flag set."""
+
+        context = copy.copy(self)
+        context.roles = copy.deepcopy(self.roles)
+        if 'admin' not in context.roles:
+            context.roles.append('admin')
+
+        context.is_admin = True
+
+        return context
+
 
 def get_admin_context(show_deleted=False):
     """Create an administrator context."""
     return RequestContext(auth_token=None,
-                          tenant=None,
+                          project_id=None,
                           is_admin=True,
                           show_deleted=show_deleted,
                           overwrite=False)
