@@ -25,13 +25,12 @@ import re
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
+from oslo_utils import timeutils
 import sqlalchemy
 from sqlalchemy import and_
 from sqlalchemy.schema import MetaData
 from sqlalchemy.sql import select
 
-from glance.common import timeutils
 from glance.i18n import _, _LE, _LI, _LW
 
 LOG = logging.getLogger(__name__)
@@ -231,7 +230,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                                  if isfile(join(metadata_path, f))
                                  and f.endswith('.json')]
     except OSError as e:
-        LOG.error(encodeutils.exception_to_unicode(e))
+        LOG.error(str(e))
         return
 
     if not json_schema_files:
@@ -253,9 +252,9 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                 metadata = json.load(json_file)
         except Exception as e:
             LOG.error(_LE("Failed to parse json file %(file_path)s while "
-                          "populating metadata due to: %(error_msg)s"),
+                          "populating metadata due to: %(exc)s"),
                       {"file_path": file,
-                       "error_msg": encodeutils.exception_to_unicode(e)})
+                       "exc": e})
             continue
 
         values = {
@@ -280,7 +279,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
 
         if db_namespace and overwrite:
             LOG.info(_LI("Overwriting namespace %s"), values['namespace'])
-            _clear_namespace_metadata(meta, db_namespace[0])
+            _clear_namespace_metadata(meta, conn, db_namespace[0])
             db_namespace = None
 
         if not db_namespace:
@@ -303,7 +302,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
             continue
         elif prefer_new:
             values.update({'updated_at': timeutils.utcnow()})
-            _update_data_in_db(namespaces_table, values,
+            _update_data_in_db(conn, namespaces_table, values,
                                namespaces_table.c.id, db_namespace[0])
 
         namespace_id = db_namespace[0]
@@ -321,7 +320,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                     meta, conn, resource_type['name'])
             elif prefer_new:
                 val = {'updated_at': timeutils.utcnow()}
-                _update_data_in_db(resource_types_table, val,
+                _update_data_in_db(conn, resource_types_table, val,
                                    resource_types_table.c.id, rt_id)
 
             values = {
@@ -338,7 +337,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                 _insert_data_to_db(conn, namespace_rt_table, values)
             elif prefer_new:
                 values.update({'updated_at': timeutils.utcnow()})
-                _update_rt_association(namespace_rt_table, values,
+                _update_rt_association(conn, namespace_rt_table, values,
                                        rt_id, namespace_id)
 
         for name, schema in metadata.get('properties', {}).items():
@@ -355,7 +354,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                 _insert_data_to_db(conn, properties_table, values)
             elif prefer_new:
                 values.update({'updated_at': timeutils.utcnow()})
-                _update_data_in_db(properties_table, values,
+                _update_data_in_db(conn, properties_table, values,
                                    properties_table.c.id, property_id)
 
         for object in metadata.get('objects', []):
@@ -373,7 +372,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                 _insert_data_to_db(conn, objects_table, values)
             elif prefer_new:
                 values.update({'updated_at': timeutils.utcnow()})
-                _update_data_in_db(objects_table, values,
+                _update_data_in_db(conn, objects_table, values,
                                    objects_table.c.id, object_id)
 
         for tag in metadata.get('tags', []):
@@ -388,7 +387,7 @@ def _populate_metadata(meta, conn, metadata_path=None, merge=False,
                 _insert_data_to_db(conn, tags_table, values)
             elif prefer_new:
                 values.update({'updated_at': timeutils.utcnow()})
-                _update_data_in_db(tags_table, values,
+                _update_data_in_db(conn, tags_table, values,
                                    tags_table.c.id, tag_id)
 
         LOG.info(_LI("File %s loaded to database."), file)
@@ -512,7 +511,7 @@ def _export_data_to_file(meta, conn, path):
             with open(file_name, 'w') as json_file:
                 json_file.write(json.dumps(values))
         except Exception as e:
-            LOG.exception(encodeutils.exception_to_unicode(e))
+            LOG.exception(str(e))
         LOG.info(_LI("Namespace %(namespace)s saved in %(file)s"), {
             'namespace': namespace_file_name, 'file': file_name})
 
