@@ -22,6 +22,7 @@ import uuid
 
 from oslo_db.sqlalchemy import models
 from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 from sqlalchemy import BigInteger
 from sqlalchemy import Boolean
 from sqlalchemy import Column
@@ -38,8 +39,6 @@ from sqlalchemy import Text
 from sqlalchemy.types import TypeDecorator
 from sqlalchemy import UniqueConstraint
 
-from glance.common import timeutils
-
 
 BASE = declarative_base()
 
@@ -70,7 +69,7 @@ class GlanceBase(models.ModelBase, models.TimestampMixin):
 
     def save(self, session=None):
         from glance.db.sqlalchemy import api as db_api
-        super(GlanceBase, self).save(session or db_api.get_session())
+        super(GlanceBase, self).save(session or db_api.session_for_write())
 
     created_at = Column(DateTime, default=lambda: timeutils.utcnow(),
                         nullable=False)
