@@ -22,9 +22,9 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import importutils
+from oslo_utils import timeutils
 
 from glance.common import exception
-from glance.common import timeutils
 from glance.i18n import _, _LE, _LI, _LW
 
 LOG = logging.getLogger(__name__)
@@ -351,7 +351,7 @@ class ImageMemberFactory(object):
 
 
 class Task(object):
-    _supported_task_type = ('import', 'api_image_import')
+    _supported_task_type = ('import', 'api_image_import', 'location_import')
 
     _supported_task_status = ('pending', 'processing', 'success', 'failure')
 
