@@ -16,13 +16,12 @@
 """
 LRU Cache for Image Data
 """
+import hashlib
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
-from oslo_utils.secretutils import md5
 from oslo_utils import units
 
 from glance.common import exception
@@ -354,7 +353,7 @@ class ImageCache(object):
 
     def cache_tee_iter(self, image_id, image_iter, image_checksum):
         try:
-            current_checksum = md5(usedforsecurity=False)
+            current_checksum = hashlib.md5(usedforsecurity=False)
 
             with self.driver.open_for_write(image_id) as cache_file:
                 for chunk in image_iter:
@@ -375,13 +374,13 @@ class ImageCache(object):
             with excutils.save_and_reraise_exception():
                 # image_iter has given us bad, (size_checked_iter has found a
                 # bad length), or corrupt data (checksum is wrong).
-                LOG.exception(encodeutils.exception_to_unicode(e))
+                LOG.exception(str(e))
         except Exception as e:
             LOG.exception(_LE("Exception encountered while tee'ing "
                               "image '%(image_id)s' into cache: %(error)s. "
                               "Continuing with response."),
                           {'image_id': image_id,
-                           'error': encodeutils.exception_to_unicode(e)})
+                           'error': e})
 
             # If no checksum provided continue responding even if
             # caching failed.
