@@ -57,33 +57,40 @@ PROJECT_READER_OR_PUBLIC_NAMESPACE = (
 # typical in OpenStack services. But following check strings offer formal
 # support for project membership and a read-only variant consistent with
 # other OpenStack services.
-ADMIN_OR_PROJECT_MEMBER = f'rule:context_is_admin or ({PROJECT_MEMBER})'
-ADMIN_OR_PROJECT_READER = f'rule:context_is_admin or ({PROJECT_READER})'
+ADMIN = 'rule:context_is_admin'
+DEFAULT = 'rule:default'
+ADMIN_OR_PROJECT_MEMBER = f'{ADMIN} or ({PROJECT_MEMBER})'
+ADMIN_OR_PROJECT_READER = f'{ADMIN} or ({PROJECT_READER})'
 ADMIN_OR_PROJECT_READER_GET_IMAGE = (
-    f'rule:context_is_admin or '
+    f'{ADMIN} or '
     f'({PROJECT_READER_OR_IMAGE_MEMBER_OR_COMMUNITY_OR_PUBLIC_OR_SHARED})'
 )
 ADMIN_OR_PROJECT_MEMBER_DOWNLOAD_IMAGE = (
-    f'rule:context_is_admin or '
+    f'{ADMIN} or '
     f'({PROJECT_MEMBER_OR_IMAGE_MEMBER_OR_COMMUNITY_OR_PUBLIC_OR_SHARED})'
 )
 ADMIN_OR_PROJECT_MEMBER_CREATE_IMAGE = (
-    f'rule:context_is_admin or ({PROJECT_MEMBER} and project_id:%(owner)s)'
+    f'{ADMIN} or ({PROJECT_MEMBER} and project_id:%(owner)s)'
 )
 ADMIN_OR_PROJECT_READER_GET_NAMESPACE = (
-    f'rule:context_is_admin or ({PROJECT_READER_OR_PUBLIC_NAMESPACE})'
+    f'{ADMIN} or ({PROJECT_READER_OR_PUBLIC_NAMESPACE})'
 )
 
 
 ADMIN_OR_SHARED_MEMBER = (
-    f'rule:context_is_admin or (role:member and {IMAGE_MEMBER_CHECK})'
+    f'{ADMIN} or (role:member and {IMAGE_MEMBER_CHECK})'
 )
 ADMIN_OR_PROJECT_READER_OR_SHARED_MEMBER = (
-    f'rule:context_is_admin or '
+    f'{ADMIN} or '
     f'role:reader and (project_id:%(project_id)s or {IMAGE_MEMBER_CHECK})'
 )
+SERVICE_OR_PROJECT_MEMBER = (
+    f'rule:service_api or ({PROJECT_MEMBER} and project_id:%(owner)s)'
+)
 
-ADMIN = 'rule:context_is_admin'
+SERVICE = 'rule:service_api'
+
+ADMIN_OR_SERVICE_ROLE = f'{ADMIN} or {SERVICE}'
 
 rules = [
     policy.RuleDefault(name='default', check_str='',
@@ -92,7 +99,7 @@ rules = [
                                    'policy in the supplied policy.json file.',
                        deprecated_rule=policy.DeprecatedRule(
                            name='default',
-                           check_str='rule:context_is_admin',
+                           check_str=ADMIN,
                            deprecated_reason='In order to allow operators to '
                            'accept the default policies from code by not '
                            'defining them in the policy file, while still '
@@ -106,6 +113,9 @@ rules = [
     policy.RuleDefault(name='context_is_admin', check_str='role:admin',
                        description='Defines the rule for the is_admin:True '
                                    'check.'),
+    policy.RuleDefault(name='service_api', check_str='service_roles:service',
+                       description='Default rule for the service-to-service '
+                       'API.'),
 ]
 
 
