@@ -14,14 +14,12 @@
 #    under the License.
 
 import calendar
+from concurrent import futures
 import time
 
-import eventlet
 from glance_store import exceptions as store_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_log import versionutils
-from oslo_utils import encodeutils
 
 from glance.common import crypt
 from glance.common import exception
@@ -31,18 +29,9 @@ import glance.db as db_api
 from glance.i18n import _, _LC, _LE, _LI, _LW
 
 LOG = logging.getLogger(__name__)
-DEPRECATED_SUBSTRING = ('is scheduled to be removed during the 2024.2 '
-                        '(Dalmatian) development cycle.')
-DEPRECATED_SCRUBBER_MSG = ('The glance scrubber has been deprecated and '
-                           + DEPRECATED_SUBSTRING)
-DEPRECATED_OPTS_MSG = ('The entire glance scrubber, including this option, '
-                       + DEPRECATED_SUBSTRING)
 
 scrubber_opts = [
     cfg.IntOpt('scrub_time', default=0, min=0,
-               deprecated_for_removal=True,
-               deprecated_since='2024.1 (Caracal)',
-               deprecated_reason=DEPRECATED_OPTS_MSG,
                help=_("""
 The amount of time, in seconds, to delay image scrubbing.
 
@@ -65,9 +54,6 @@ Related options:
 
 """)),
     cfg.IntOpt('scrub_pool_size', default=1, min=1,
-               deprecated_for_removal=True,
-               deprecated_since='2024.1 (Caracal)',
-               deprecated_reason=DEPRECATED_OPTS_MSG,
                help=_("""
 The size of thread pool to be used for scrubbing images.
 
@@ -86,9 +72,6 @@ Related options:
 
 """)),
     cfg.BoolOpt('delayed_delete', default=False,
-                deprecated_for_removal=True,
-                deprecated_since='2024.1 (Caracal)',
-                deprecated_reason=DEPRECATED_OPTS_MSG,
                 help=_("""
 Turn on/off delayed delete.
 
@@ -120,9 +103,6 @@ Related options:
 
 scrubber_cmd_opts = [
     cfg.IntOpt('wakeup_time', default=300, min=0,
-               deprecated_for_removal=True,
-               deprecated_since='2024.1 (Caracal)',
-               deprecated_reason=DEPRECATED_OPTS_MSG,
                help=_("""
 Time interval, in seconds, between scrubber runs in daemon mode.
 
@@ -149,9 +129,6 @@ scrubber_cmd_cli_opts = [
     cfg.BoolOpt('daemon',
                 short='D',
                 default=False,
-                deprecated_for_removal=True,
-                deprecated_since='2024.1 (Caracal)',
-                deprecated_reason=DEPRECATED_OPTS_MSG,
                 help=_("""
 Run scrubber as a daemon.
 
@@ -175,9 +152,6 @@ Related options:
 """)),
     cfg.StrOpt('restore',
                metavar='<IMAGE_ID>',
-               deprecated_for_removal=True,
-               deprecated_since='2024.1 (Caracal)',
-               deprecated_reason=DEPRECATED_OPTS_MSG,
                help=_("""
 Restore the image status from 'pending_delete' to 'active'.
 
@@ -313,40 +287,42 @@ def get_scrub_queue():
 
 class Daemon(object):
     def __init__(self, wakeup_time=300, threads=100):
-        versionutils.report_deprecated_feature(LOG, DEPRECATED_SCRUBBER_MSG)
         LOG.info(_LI("Starting Daemon: wakeup_time=%(wakeup_time)s "
                      "threads=%(threads)s"),
                  {'wakeup_time': wakeup_time, 'threads': threads})
         self.wakeup_time = wakeup_time
-        self.event = eventlet.event.Event()
         # This pool is used for periodic instantiation of scrubber
-        self.daemon_pool = eventlet.greenpool.GreenPool(threads)
+        self.executor = futures.ThreadPoolExecutor(max_workers=threads)
 
     def start(self, application):
         self._run(application)
 
     def wait(self):
         try:
-            self.event.wait()
+            while True:
+                # Keep the daemon alive; can be replaced with more
+                # sophisticated handling.
+                time.sleep(1)
         except KeyboardInterrupt:
-            msg = _LI("Daemon Shutdown on KeyboardInterrupt")
-            LOG.info(msg)
+            LOG.info(_LI("Daemon Shutdown on KeyboardInterrupt"))
 
     def _run(self, application):
         LOG.debug("Running application")
-        self.daemon_pool.spawn_n(application.run, self.event)
-        eventlet.spawn_after(self.wakeup_time, self._run, application)
+        future: futures.Future = self.executor.submit(application.run)
+        # Schedule next run
         LOG.debug("Next run scheduled in %s seconds", self.wakeup_time)
+        time.sleep(self.wakeup_time)
+        future.add_done_callback(self._run(application))
 
 
 class Scrubber(object):
     def __init__(self, store_api):
-        versionutils.report_deprecated_feature(LOG, DEPRECATED_SCRUBBER_MSG)
         LOG.info(_LI("Initializing scrubber"))
         self.store_api = store_api
         self.admin_context = context.get_admin_context(show_deleted=True)
         self.db_queue = get_scrub_queue()
-        self.pool = eventlet.greenpool.GreenPool(CONF.scrub_pool_size)
+        self.executor = futures.ThreadPoolExecutor(
+            max_workers=CONF.scrub_pool_size)
 
     def _get_delete_jobs(self):
         try:
@@ -354,8 +330,7 @@ class Scrubber(object):
         except Exception as err:
             # Note(dharinic): spawn_n, in Daemon mode will log the
             # exception raised. Otherwise, exit 1 will occur.
-            msg = (_LC("Can not get scrub jobs from queue: %s") %
-                   encodeutils.exception_to_unicode(err))
+            msg = _LC("Can not get scrub jobs from queue: %s") % err
             LOG.critical(msg)
             raise exception.FailedToGetScrubberJobs()
 
@@ -378,31 +353,28 @@ class Scrubber(object):
         delete_jobs = self._get_delete_jobs()
 
         if delete_jobs:
-            list(self.pool.starmap(self._scrub_image, delete_jobs.items()))
+            self.executor.map(self._scrub_image, delete_jobs.items())
 
-    def _scrub_image(self, image_id, delete_jobs):
+    def _scrub_image(self, delete_jobs):
         if len(delete_jobs) == 0:
             return
 
+        image_id, loc_details = delete_jobs
         LOG.info(_LI("Scrubbing image %(id)s from %(count)d locations."),
                  {'id': image_id, 'count': len(delete_jobs)})
-
         success = True
-        if CONF.enabled_backends:
-            for img_id, loc_id, uri, backend in delete_jobs:
-                try:
-                    self._delete_image_location_from_backend(img_id, loc_id,
-                                                             uri,
-                                                             backend=backend)
-                except Exception:
-                    success = False
-        else:
-            for img_id, loc_id, uri in delete_jobs:
-                try:
-                    self._delete_image_location_from_backend(img_id, loc_id,
-                                                             uri)
-                except Exception:
-                    success = False
+        for item in loc_details:
+            try:
+                if CONF.enabled_backends:
+                    img_id, loc_id, uri, backend = item
+                    self._delete_image_location_from_backend(
+                        img_id, loc_id, uri, backend=backend)
+                else:
+                    img_id, loc_id, uri = item
+                    self._delete_image_location_from_backend(
+                        img_id, loc_id, uri)
+            except Exception:
+                success = False
 
         if success:
             image = db_api.get_api().image_get(self.admin_context, image_id)
@@ -420,7 +392,7 @@ class Scrubber(object):
         try:
             LOG.debug("Scrubbing image %s from a location.", image_id)
             try:
-                if CONF.enabled_backends:
+                if backend:
                     self.store_api.delete(uri, backend, self.admin_context)
                 else:
                     self.store_api.delete_from_backend(uri, self.admin_context)
@@ -438,8 +410,7 @@ class Scrubber(object):
         except Exception as e:
             LOG.error(_LE("Unable to scrub image %(id)s from a location. "
                           "Reason: %(exc)s "),
-                      {'id': image_id,
-                       'exc': encodeutils.exception_to_unicode(e)})
+                      {'id': image_id, 'exc': e})
             raise
 
     def revert_image_status(self, image_id):
