@@ -30,11 +30,11 @@ import platform
 import shutil
 import signal
 import socket
-import subprocess
 import sys
 import tempfile
 from testtools import content as ttc
 import textwrap
+import threading
 import time
 from unittest import mock
 import urllib.parse as urlparse
@@ -42,7 +42,6 @@ import uuid
 
 import fixtures
 import glance_store
-from os_win import utilsfactory as os_win_utilsfactory
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 import testtools
@@ -52,18 +51,14 @@ from glance.common import config
 from glance.common import utils
 from glance.common import wsgi
 from glance.db.sqlalchemy import api as db_api
+from glance import housekeeping
 from glance import tests as glance_tests
 from glance.tests import utils as test_utils
 
 execute, get_unused_port = test_utils.execute, test_utils.get_unused_port
 tracecmd_osmap = {'Linux': 'strace', 'FreeBSD': 'truss'}
 
-if os.name == 'nt':
-    SQLITE_CONN_TEMPLATE = 'sqlite:///%s/tests.sqlite'
-else:
-    SQLITE_CONN_TEMPLATE = 'sqlite:////%s/tests.sqlite'
-
-
+SQLITE_CONN_TEMPLATE = 'sqlite:////%s/tests.sqlite'
 CONF = cfg.CONF
 
 
@@ -301,79 +296,7 @@ class PosixServer(BaseServer):
         return (rc, '', '')
 
 
-class Win32Server(BaseServer):
-    def __init__(self, *args, **kwargs):
-        super(Win32Server, self).__init__(*args, **kwargs)
-
-        self._processutils = os_win_utilsfactory.get_processutils()
-
-    def start(self, expect_exit=True, expected_exitcode=0, **kwargs):
-        """
-        Starts the server.
-
-        Any kwargs passed to this method will override the configuration
-        value in the conf file used in starting the servers.
-        """
-
-        # Ensure the configuration file is written
-        self.write_conf(**kwargs)
-
-        self.create_database()
-
-        cmd = ("%(server_module)s --config-file %(conf_file_name)s"
-               % {"server_module": self.server_module,
-                  "conf_file_name": self.conf_file_name})
-        cmd = "%s -m %s" % (sys.executable, cmd)
-
-        # Passing socket objects on Windows is a bit more cumbersome.
-        # We don't really have to do it.
-        if self.sock:
-            self.sock.close()
-            self.sock = None
-
-        self.process = subprocess.Popen(
-            cmd,
-            env=self.exec_env)
-        self.process_pid = self.process.pid
-
-        try:
-            self.job_handle = self._processutils.kill_process_on_job_close(
-                self.process_pid)
-        except Exception:
-            # Could not associate child process with a job, killing it.
-            self.process.kill()
-            raise
-
-        self.stop_kill = not expect_exit
-        if self.pid_file:
-            pf = open(self.pid_file, 'w')
-            pf.write('%d\n' % self.process_pid)
-            pf.close()
-
-        rc = 0
-        if expect_exit:
-            self.process.communicate()
-            rc = self.process.returncode
-
-        return (rc, '', '')
-
-    def stop(self):
-        """
-        Spin down the server.
-        """
-        if not self.process_pid:
-            raise Exception('Server "%s" process not running.'
-                            % self.server_name)
-
-        if self.stop_kill:
-            self.process.terminate()
-        return (0, '', '')
-
-
-if os.name == 'nt':
-    Server = Win32Server
-else:
-    Server = PosixServer
+Server = PosixServer
 
 
 class ApiServer(Server):
@@ -396,7 +319,6 @@ class ApiServer(Server):
         self.image_size_cap = 1099511627776
         self.delayed_delete = delayed_delete
         self.workers = 0
-        self.scrub_time = 5
         self.image_cache_dir = os.path.join(self.test_dir,
                                             'cache')
         self.image_cache_driver = 'sqlite'
@@ -431,7 +353,6 @@ log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
 workers = %(workers)s
-scrub_time = %(scrub_time)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
 show_image_direct_url = %(show_image_direct_url)s
@@ -581,7 +502,6 @@ class ApiServerForMultipleBackend(Server):
         self.image_size_cap = 1099511627776
         self.delayed_delete = delayed_delete
         self.workers = 0
-        self.scrub_time = 5
         self.image_cache_dir = os.path.join(self.test_dir,
                                             'cache')
         self.image_cache_driver = 'sqlite'
@@ -613,7 +533,6 @@ log_file = %(log_file)s
 image_size_cap = %(image_size_cap)d
 delayed_delete = %(delayed_delete)s
 workers = %(workers)s
-scrub_time = %(scrub_time)s
 image_cache_dir = %(image_cache_dir)s
 image_cache_driver = %(image_cache_driver)s
 show_image_direct_url = %(show_image_direct_url)s
@@ -745,58 +664,6 @@ allowed_origin=http://valid.example.com
 """
 
 
-class ScrubberDaemon(Server):
-    """
-    Server object that starts/stops/manages the Scrubber server
-    """
-
-    def __init__(self, test_dir, policy_file, daemon=False, **kwargs):
-        # NOTE(jkoelker): Set the port to 0 since we actually don't listen
-        super(ScrubberDaemon, self).__init__(test_dir, 0)
-        self.server_name = 'scrubber'
-        self.server_module = 'glance.cmd.%s' % self.server_name
-        self.daemon = daemon
-
-        self.image_dir = os.path.join(self.test_dir, "images")
-        self.scrub_time = 5
-        self.pid_file = os.path.join(self.test_dir, "scrubber.pid")
-        self.log_file = os.path.join(self.test_dir, "scrubber.log")
-        self.metadata_encryption_key = "012345678901234567890123456789ab"
-        self.lock_path = self.test_dir
-
-        default_sql_connection = SQLITE_CONN_TEMPLATE % self.test_dir
-        self.sql_connection = os.environ.get('GLANCE_TEST_SQL_CONNECTION',
-                                             default_sql_connection)
-        self.policy_file = policy_file
-        self.policy_default_rule = 'default'
-
-        self.conf_base = """[DEFAULT]
-debug = %(debug)s
-log_file = %(log_file)s
-daemon = %(daemon)s
-wakeup_time = 2
-scrub_time = %(scrub_time)s
-metadata_encryption_key = %(metadata_encryption_key)s
-lock_path = %(lock_path)s
-sql_idle_timeout = 3600
-[database]
-connection = %(sql_connection)s
-[glance_store]
-filesystem_store_datadir=%(image_dir)s
-[oslo_policy]
-policy_file = %(policy_file)s
-policy_default_rule = %(policy_default_rule)s
-"""
-
-    def start(self, expect_exit=True, expected_exitcode=0, **kwargs):
-        if 'daemon' in kwargs:
-            expect_exit = False
-        return super(ScrubberDaemon, self).start(
-            expect_exit=expect_exit,
-            expected_exitcode=expected_exitcode,
-            **kwargs)
-
-
 class FunctionalTest(test_utils.BaseTestCase):
 
     """
@@ -814,10 +681,6 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
-        # NOTE: Scrubber is enabled by default for the functional tests.
-        # Please disable it by explicitly setting 'self.include_scrubber' to
-        # False in the test SetUps that do not require Scrubber to run.
-        self.include_scrubber = True
 
         # The clients will try to connect to this address. Let's make sure
         # we're not using the default '0.0.0.0'
@@ -842,10 +705,7 @@ class FunctionalTest(test_utils.BaseTestCase):
                                     self.policy_file,
                                     sock=api_sock)
 
-        self.scrubber_daemon = ScrubberDaemon(self.test_dir, self.policy_file)
-
-        self.pid_files = [self.api_server.pid_file,
-                          self.scrubber_daemon.pid_file]
+        self.pid_files = [self.api_server.pid_file]
         self.files_to_destroy = []
         self.launched_servers = []
         # Keep track of servers we've logged so we don't double-log them.
@@ -909,8 +769,7 @@ class FunctionalTest(test_utils.BaseTestCase):
         # server is dead.  This eliminates the possibility of a race
         # between a child process listening on a port actually dying
         # and a new process being started
-        servers = [self.api_server,
-                   self.scrubber_daemon]
+        servers = [self.api_server]
         for s in servers:
             try:
                 s.stop()
@@ -1004,12 +863,6 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         self.start_with_retry(self.api_server, 'api_port', 3, **kwargs)
 
-        if self.include_scrubber:
-            exitcode, out, err = self.scrubber_daemon.start(**kwargs)
-            self.assertEqual(0, exitcode,
-                             "Failed to spin up the Scrubber daemon. "
-                             "Got: %s" % err)
-
     def ping_server(self, port):
         """
         Simple ping on the port. If responsive, return True, else
@@ -1123,8 +976,6 @@ class FunctionalTest(test_utils.BaseTestCase):
 
         # Spin down the API server
         self.stop_server(self.api_server)
-        if self.include_scrubber:
-            self.stop_server(self.scrubber_daemon)
 
     def copy_data_file(self, file_name, dst_dir):
         src_file_name = os.path.join('glance/tests/etc', file_name)
@@ -1160,10 +1011,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
 
         self.api_protocol = 'http'
         self.api_port, api_sock = test_utils.get_unused_port_and_socket()
-        # NOTE: Scrubber is enabled by default for the functional tests.
-        # Please disable it by explicitly setting 'self.include_scrubber' to
-        # False in the test SetUps that do not require Scrubber to run.
-        self.include_scrubber = True
 
         self.tracecmd = tracecmd_osmap.get(platform.system())
 
@@ -1182,10 +1029,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         self.api_server_multiple_backend = ApiServerForMultipleBackend(
             self.test_dir, self.api_port, self.policy_file, sock=api_sock)
 
-        self.scrubber_daemon = ScrubberDaemon(self.test_dir, self.policy_file)
-
-        self.pid_files = [self.api_server_multiple_backend.pid_file,
-                          self.scrubber_daemon.pid_file]
+        self.pid_files = [self.api_server_multiple_backend.pid_file]
         self.files_to_destroy = []
         self.launched_servers = []
         # Keep track of servers we've logged so we don't double-log them.
@@ -1251,8 +1095,7 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         # server is dead.  This eliminates the possibility of a race
         # between a child process listening on a port actually dying
         # and a new process being started
-        servers = [self.api_server_multiple_backend,
-                   self.scrubber_daemon]
+        servers = [self.api_server_multiple_backend]
         for s in servers:
             try:
                 s.stop()
@@ -1347,12 +1190,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
         self.start_with_retry(self.api_server_multiple_backend,
                               'api_port', 3, **kwargs)
 
-        if self.include_scrubber:
-            exitcode, out, err = self.scrubber_daemon.start(**kwargs)
-            self.assertEqual(0, exitcode,
-                             "Failed to spin up the Scrubber daemon. "
-                             "Got: %s" % err)
-
     def ping_server(self, port):
         """
         Simple ping on the port. If responsive, return True, else
@@ -1466,8 +1303,6 @@ class MultipleBackendFunctionalTest(test_utils.BaseTestCase):
 
         # Spin down the API
         self.stop_server(self.api_server_multiple_backend)
-        if self.include_scrubber:
-            self.stop_server(self.scrubber_daemon)
 
     def copy_data_file(self, file_name, dst_dir):
         src_file_name = os.path.join('glance/tests/etc', file_name)
@@ -1531,6 +1366,41 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             # we db_sync.
             test_utils.db_sync(engine=engine)
 
+    def setup_scrubber_conf(self, daemon=False, wakeup_time=300,
+                            scrub_pool_size=12):
+        self.scrubber_conf = os.path.join(
+            self.test_dir, 'glance-scrubber.conf')
+        db_file = 'sqlite:///%s/test.db' % self.test_dir
+        with open(self.scrubber_conf, 'w') as f:
+            f.write(textwrap.dedent("""
+            [DEFAULT]
+            enabled_backends=store1:file,store2:file,store3:file
+            daemon=%(daemon)s
+            wakeup_time=%(wakeup_time)s
+            scrub_pool_size=%(scrub_pool_size)s
+            [database]
+            connection=%(connection)s
+            [store1]
+            filesystem_store_datadir=%(store1)s
+            [store2]
+            filesystem_store_datadir=%(store2)s
+            [store3]
+            filesystem_store_datadir=%(store3)s
+            [os_glance_staging_dir]
+            filesystem_store_datadir=%(staging)s
+            [glance_store]
+            default_backend=store1
+            """) % {
+                "daemon": daemon,
+                "wakeup_time": wakeup_time,
+                "scrub_pool_size": scrub_pool_size,
+                "connection": db_file,
+                "store1": self._store_dir('store1'),
+                "store2": self._store_dir('store2'),
+                "store3": self._store_dir('store3'),
+                "staging": self._store_dir('staging'),
+            })
+
     def setup_simple_paste(self):
         """Setup a very simple no-auth paste pipeline.
 
@@ -1540,12 +1410,20 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         self.paste_config = os.path.join(self.test_dir, 'glance-api-paste.ini')
         with open(self.paste_config, 'w') as f:
             f.write(textwrap.dedent("""
+            [filter:healthcheck]
+            paste.filter_factory = oslo_middleware:Healthcheck.factory
+            path = /healthcheck
+            backends = disable_by_file
+            disable_by_file_path = /tmp/test_path
             [filter:context]
             paste.filter_factory = glance.api.middleware.context:\
                 ContextMiddleware.factory
             [filter:fakeauth]
             paste.filter_factory = glance.tests.utils:\
                 FakeAuthMiddleware.factory
+            [filter:cors]
+            paste.filter_factory = oslo_middleware.cors:filter_factory
+            allowed_origin=http://valid.example.com
             [filter:cache]
             paste.filter_factory = glance.api.middleware.cache:\
             CacheFilter.factory
@@ -1554,10 +1432,14 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             CacheManageFilter.factory
             [pipeline:glance-api-cachemanagement]
             pipeline = context cache cachemanage rootapp
+            [pipeline:glance-api-cors]
+            pipeline = cors context rootapp
             [pipeline:glance-api-caching]
             pipeline = context cache rootapp
             [pipeline:glance-api]
-            pipeline = context rootapp
+            pipeline = healthcheck context rootapp
+            [pipeline:glance-api-fake]
+            pipeline = fakeauth context rootapp
             [composite:rootapp]
             paste.composite_factory = glance.api:root_app_factory
             /v2: apiv2app
@@ -1568,6 +1450,24 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
     def _store_dir(self, store):
         return os.path.join(self.test_dir, store)
 
+    def setup_single_store(self):
+        """Configures single backend.
+
+        This configures the API with one file-backed store
+        as well as node_staging_uri for imports.
+
+        """
+        glance_store.register_opts(CONF)
+        self.config(filesystem_store_datadir=self._store_dir('store1'),
+                    group='glance_store')
+        node_staging_uri = 'file://%s' % os.path.join(
+            self.test_dir, 'staging')
+        utils.safe_mkdirs(node_staging_uri[7:])
+        self.config(node_staging_uri=node_staging_uri)
+        self.config(default_store='file', group='glance_store')
+        glance_store.create_stores(CONF)
+        glance_store.verify_default_store()
+
     def setup_stores(self):
         """Configures multiple backend stores.
 
@@ -1597,14 +1497,37 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
                                          reserved_stores=wsgi.RESERVED_STORES)
         glance_store.verify_store()
 
-    def setUp(self):
+    def setUp(self, single_store=False, bypass_headers=False):
         super(SynchronousAPIBase, self).setUp()
+        self.bypass_headers = bypass_headers
 
         self.setup_database()
         self.setup_simple_paste()
-        self.setup_stores()
+        if single_store:
+            self.setup_single_store()
+        else:
+            self.setup_stores()
 
-    def start_server(self, enable_cache=True, set_worker_url=True):
+        conf_dir = os.path.join(self.test_dir, 'etc')
+        utils.safe_mkdirs(conf_dir)
+        self.copy_data_file('schema-image.json', conf_dir)
+        self.copy_data_file('property-protections.conf', conf_dir)
+        self.copy_data_file('property-protections-policies.conf', conf_dir)
+        self.property_file_roles = os.path.join(conf_dir,
+                                                'property-protections.conf')
+        property_policies = 'property-protections-policies.conf'
+        self.property_file_policies = os.path.join(conf_dir,
+                                                   property_policies)
+
+    def copy_data_file(self, file_name, dst_dir):
+        src_file_name = os.path.join('glance/tests/etc', file_name)
+        shutil.copy(src_file_name, dst_dir)
+        dst_file_name = os.path.join(dst_dir, file_name)
+        return dst_file_name
+
+    def start_server(self, enable_cache=True, set_worker_url=True,
+                     use_fake_auth=False, run_staging_cleaner=False,
+                     enable_cors=False):
         """Builds and "starts" the API server.
 
         Note that this doesn't actually "start" anything like
@@ -1617,9 +1540,25 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             root_app = 'glance-api-cachemanagement'
             self.config(image_cache_dir=self._store_dir('cache'))
 
+        if enable_cors:
+            root_app = 'glance-api-cors'
+
+        if use_fake_auth:
+            root_app = 'glance-api-fake'
+
         if set_worker_url:
             self.config(worker_self_reference_url='http://workerx')
 
+        if run_staging_cleaner:
+            cleaner = housekeeping.StagingStoreCleaner(db_api)
+            # NOTE(danms): Start thread as a daemon. It is still a
+            # single-shot, but this will not block our shutdown if it is
+            # running.
+            cleanup_thread = threading.Thread(
+                target=cleaner.clean_orphaned_staging_residue,
+                daemon=True)
+            cleanup_thread.start()
+
         self.api = config.load_paste_app(root_app,
                                          conf_file=self.paste_config)
         self.config(enforce_new_defaults=True,
@@ -1627,6 +1566,28 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         self.config(enforce_scope=True,
                     group='oslo_policy')
 
+    def start_scrubber(self, daemon=False, wakeup_time=300,
+                       restore=None, raise_error=True):
+        self.setup_scrubber_conf(daemon=daemon, wakeup_time=wakeup_time)
+        exe_cmd = f"{sys.executable} -m glance.cmd.scrubber"
+
+        # Modify command based on restore and daemon flags
+        if restore:
+            exe_cmd += f" --restore {restore}"
+        if daemon:
+            exe_cmd += " --daemon"
+
+        # Prepare the final command string with the config directory
+        cmd = f"{exe_cmd} --config-dir {self.test_dir}"
+
+        # Determine if we need to return the process object
+        expect_exit = not daemon
+        return_process = daemon
+
+        # Execute the command and return the result
+        return execute(cmd, raise_error=raise_error, expect_exit=expect_exit,
+                       return_process=return_process)
+
     def _headers(self, custom_headers=None):
         base_headers = {
             'X-Identity-Status': 'Confirmed',
@@ -1656,7 +1617,8 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
                           stream for the request (overrides @data)
         :returns: A webob.Response object
         """
-        headers = self._headers(headers)
+        if not self.bypass_headers:
+            headers = self._headers(headers)
         req = webob.Request.blank(url, method=method,
                                   headers=headers)
         if json and not data:
@@ -1667,14 +1629,18 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
             req.body_file = body_file
         return self.api(req)
 
-    def api_get(self, url, headers=None):
+    def api_get(self, url, headers=None, data=None, json=None):
         """Perform a GET request against the API.
 
         :param url: The *path* part of the URL to call (i.e. /v2/images)
         :param headers: Optional updates to the default set of headers
+        :param data: Optional bytes data payload to send (overrides @json)
+        :param json: Optional dict structure to be jsonified and sent as
+                     the payload (mutually exclusive with @data)
         :returns: A webob.Response object
         """
-        return self.api_request('GET', url, headers=headers)
+        return self.api_request('GET', url, headers=headers,
+                                data=data, json=json)
 
     def api_post(self, url, headers=None, data=None, json=None,
                  body_file=None):
@@ -1710,16 +1676,20 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
                                 data=data, json=json,
                                 body_file=body_file)
 
-    def api_delete(self, url, headers=None):
+    def api_delete(self, url, headers=None, data=None, json=None):
         """Perform a DELETE request against the API.
 
         :param url: The *path* part of the URL to call (i.e. /v2/images)
         :param headers: Optional updates to the default set of headers
+        :param data: Optional bytes data payload to send (overrides @json)
+        :param json: Optional dict structure to be jsonified and sent as
+                     the payload (mutually exclusive with @data)
         :returns: A webob.Response object
         """
-        return self.api_request('DELETE', url, headers=headers)
+        return self.api_request('DELETE', url, headers=headers,
+                                data=data, json=json)
 
-    def api_patch(self, url, *patches, headers=None):
+    def api_patch(self, url, patches, headers=None):
         """Perform a PATCH request against the API.
 
         :param url: The *path* part of the URL to call (i.e. /v2/images)
@@ -1727,12 +1697,14 @@ class SynchronousAPIBase(test_utils.BaseTestCase):
         :param headers: Optional updates to the default set of headers
         :returns: A webob.Response object
         """
+        json = patches if isinstance(patches, list) else list(patches)
+
         if not headers:
             headers = {}
         headers['Content-Type'] = \
             'application/openstack-images-v2.1-json-patch'
         return self.api_request('PATCH', url, headers=headers,
-                                json=list(patches))
+                                json=json)
 
     def _import_copy(self, image_id, stores, headers=None):
         """Do an import of image_id to the given stores."""
