@@ -12,11 +12,12 @@
 
 import datetime
 
-from oslo_db.sqlalchemy import test_base
+from oslo_db.sqlalchemy import test_fixtures
 from oslo_db.sqlalchemy import utils as db_utils
 
 from glance.db.sqlalchemy.alembic_migrations import data_migrations
 from glance.tests.functional.db import test_migrations
+import glance.tests.utils as test_utils
 
 
 class TestOcataContract01Mixin(test_migrations.AlembicMigrationsMixin):
@@ -41,7 +42,8 @@ class TestOcataContract01Mixin(test_migrations.AlembicMigrationsMixin):
                            min_disk=0,
                            min_ram=0,
                            id='public_id_before_expand')
-        images.insert().values(public_temp).execute()
+        with engine.connect() as conn, conn.begin():
+            conn.execute(images.insert().values(public_temp))
 
         # inserting a private image record
         shared_temp = dict(deleted=False,
@@ -51,7 +53,8 @@ class TestOcataContract01Mixin(test_migrations.AlembicMigrationsMixin):
                            min_disk=0,
                            min_ram=0,
                            id='private_id_before_expand')
-        images.insert().values(shared_temp).execute()
+        with engine.connect() as conn, conn.begin():
+            conn.execute(images.insert().values(shared_temp))
 
         data_migrations.migrate(engine=engine, release='ocata')
 
@@ -62,6 +65,9 @@ class TestOcataContract01Mixin(test_migrations.AlembicMigrationsMixin):
         self.assertIn('visibility', images.c)
 
 
-class TestOcataContract01MySQL(TestOcataContract01Mixin,
-                               test_base.MySQLOpportunisticTestCase):
-    pass
+class TestOcataContract01MySQL(
+    TestOcataContract01Mixin,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_utils.BaseTestCase,
+):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
