@@ -15,11 +15,11 @@
 
 """Functional test case that tests logging output"""
 
+import http.client as http
 import os
 import stat
 
 import httplib2
-from six.moves import http_client as http
 
 from glance.tests import functional
 
@@ -36,7 +36,7 @@ class TestLogging(functional.FunctionalTest):
         self.start_servers()
 
         # The default functional test case has both debug on. Let's verify
-        # that debug statements appear in both the API and registry logs.
+        # that debug statements appear in the API logs.
 
         self.assertTrue(os.path.exists(self.api_server.log_file))
 
@@ -45,13 +45,6 @@ class TestLogging(functional.FunctionalTest):
 
         self.assertIn('DEBUG glance', api_log_out)
 
-        self.assertTrue(os.path.exists(self.registry_server.log_file))
-
-        with open(self.registry_server.log_file, 'r') as freg:
-            registry_log_out = freg.read()
-
-        self.assertIn('DEBUG glance', registry_log_out)
-
         self.stop_servers()
 
     def test_no_debug(self):
@@ -67,14 +60,6 @@ class TestLogging(functional.FunctionalTest):
             api_log_out = f.read()
 
         self.assertNotIn('DEBUG glance', api_log_out)
-
-        self.assertTrue(os.path.exists(self.registry_server.log_file))
-
-        with open(self.registry_server.log_file, 'r') as freg:
-            registry_log_out = freg.read()
-
-        self.assertNotIn('DEBUG glance', registry_log_out)
-
         self.stop_servers()
 
     def assertNotEmptyFile(self, path):
@@ -85,12 +70,6 @@ class TestLogging(functional.FunctionalTest):
         """
         Test that we notice when our log file has been rotated
         """
-
-        # Moving in-use files is not supported on Windows.
-        # The log handler itself may be configured to rotate files.
-        if os.name == 'nt':
-            raise self.skipException("Unsupported platform.")
-
         self.cleanup()
         self.start_servers()
 
