@@ -13,13 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import http.client as http
 import os
 import re
 import time
 
 import psutil
 import requests
-from six.moves import http_client as http
 
 from glance.tests import functional
 from glance.tests.utils import execute
@@ -47,7 +47,6 @@ class TestReload(functional.FunctionalTest):
         super(TestReload, self).setUp()
         self.cleanup()
         self.workers = 1
-        self.include_scrubber = False
 
     def tearDown(self):
         if not self.disabled:
@@ -79,12 +78,7 @@ class TestReload(functional.FunctionalTest):
         pid = None
         pid = self._get_parent(server)
         process = psutil.Process(pid)
-        try:
-            # psutils version >= 2
-            children = process.children()
-        except AttributeError:
-            # psutils version < 2
-            children = process.get_children()
+        children = process.children()
         pids = set()
         for child in children:
             pids.add(child.pid)
@@ -99,9 +93,6 @@ class TestReload(functional.FunctionalTest):
         conf_filepath = os.path.join(conf_dir, '%s.conf' % service)
         return conf_filepath
 
-    def _url(self, protocol, path):
-        return '%s://127.0.0.1:%d%s' % (protocol, self.api_port, path)
-
     def test_reload(self):
         """Test SIGHUP picks up new config values"""
         def check_pids(pre, post=None, workers=2):
@@ -121,7 +112,7 @@ class TestReload(functional.FunctionalTest):
         pre_pids = {}
         post_pids = {}
 
-        path = self._url('http', '/')
+        path = self._url('/')
         response = requests.get(path)
         self.assertEqual(http.MULTIPLE_CHOICES, response.status_code)
         del response  # close socket so that process audit is reliable
@@ -159,7 +150,7 @@ class TestReload(functional.FunctionalTest):
             if check_pids(pre_pids['api'], post_pids['api']):
                 break
 
-        path = self._url('http', '/')
+        path = self._url('/')
         response = requests.get(path)
         self.assertEqual(http.MULTIPLE_CHOICES, response.status_code)
         del response
