@@ -26,19 +26,30 @@ from oslo_db import options
 import glance.common.client
 from glance.common import config
 import glance.db.sqlalchemy.api
-import glance.registry.client.v1.client
 from glance import tests as glance_tests
 from glance.tests import utils as test_utils
 
 
 TESTING_API_PASTE_CONF = """
-[pipeline:glance-api]
+[composite:glance-api]
+paste.composite_factory = glance.api:root_app_factory
+/: api
+
+[pipeline: api]
 pipeline = versionnegotiation gzip unauthenticated-context rootapp
 
-[pipeline:glance-api-caching]
+[composite:glance-api-caching]
+paste.composite_factory = glance.api:root_app_factory
+/: api-caching
+
+[pipeline: api-caching]
 pipeline = versionnegotiation gzip unauthenticated-context cache rootapp
 
-[pipeline:glance-api-cachemanagement]
+[composite:glance-api-cachemanagement]
+paste.composite_factory = glance.api:root_app_factory
+/: api-cachemanagement
+
+[pipeline: api-cachemanagement]
 pipeline =
     versionnegotiation
     gzip
@@ -47,24 +58,28 @@ pipeline =
     cache_manage
     rootapp
 
-[pipeline:glance-api-fakeauth]
+[composite:glance-api-fakeauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-fakeauth
+
+[pipeline: api-fakeauth]
 pipeline = versionnegotiation gzip fakeauth context rootapp
 
-[pipeline:glance-api-noauth]
+[composite:glance-api-noauth]
+paste.composite_factory = glance.api:root_app_factory
+/: api-noauth
+
+[pipeline: api-noauth]
 pipeline = versionnegotiation gzip context rootapp
 
 [composite:rootapp]
 paste.composite_factory = glance.api:root_app_factory
 /: apiversions
-/v1: apiv1app
 /v2: apiv2app
 
 [app:apiversions]
 paste.app_factory = glance.api.versions:create_resource
 
-[app:apiv1app]
-paste.app_factory = glance.api.v1.router:API.factory
-
 [app:apiv2app]
 paste.app_factory = glance.api.v2.router:API.factory
 
@@ -93,27 +108,6 @@ paste.filter_factory =
 paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
 """
 
-TESTING_REGISTRY_PASTE_CONF = """
-[pipeline:glance-registry]
-pipeline = unauthenticated-context registryapp
-
-[pipeline:glance-registry-fakeauth]
-pipeline = fakeauth context registryapp
-
-[app:registryapp]
-paste.app_factory = glance.registry.api.v1:API.factory
-
-[filter:context]
-paste.filter_factory = glance.api.middleware.context:ContextMiddleware.factory
-
-[filter:unauthenticated-context]
-paste.filter_factory =
- glance.api.middleware.context:UnauthenticatedContextMiddleware.factory
-
-[filter:fakeauth]
-paste.filter_factory = glance.tests.utils:FakeAuthMiddleware.factory
-"""
-
 CONF = cfg.CONF
 
 
@@ -126,13 +120,6 @@ class ApiTest(test_utils.BaseTestCase):
         self._setup_database()
         self._setup_stores()
         self._setup_property_protection()
-        self.glance_registry_app = self._load_paste_app(
-            'glance-registry',
-            flavor=getattr(self, 'registry_flavor', ''),
-            conf=getattr(self, 'registry_paste_conf',
-                         TESTING_REGISTRY_PASTE_CONF),
-        )
-        self._connect_registry_client()
         self.glance_api_app = self._load_paste_app(
             'glance-api',
             flavor=getattr(self, 'api_flavor', ''),
@@ -203,15 +190,6 @@ class ApiTest(test_utils.BaseTestCase):
         return config.load_paste_app(name, flavor=flavor,
                                      conf_file=conf_file_path)
 
-    def _connect_registry_client(self):
-        def get_connection_type(self2):
-            def wrapped(*args, **kwargs):
-                return test_utils.HttplibWsgiAdapter(self.glance_registry_app)
-            return wrapped
-
-        self.mock_object(glance.common.client.BaseClient,
-                         'get_connection_type', get_connection_type)
-
     def tearDown(self):
         glance.db.sqlalchemy.api.clear_db_env()
         super(ApiTest, self).tearDown()
