@@ -13,15 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import io
 import json
 import os
 from unittest import mock
+import urllib
 
 import glance_store
 from oslo_concurrency import processutils as putils
 from oslo_config import cfg
-import six
-from six.moves import urllib
 from taskflow import task
 from taskflow.types import failure
 
@@ -74,7 +74,7 @@ class TestImportTask(test_utils.BaseTestCase):
         self.task_factory = domain.TaskFactory()
         self.img_factory = self.gateway.get_image_factory(self.context)
         self.image = self.img_factory.new_image(image_id=UUID1,
-                                                disk_format='qcow2',
+                                                disk_format='raw',
                                                 container_format='bare')
 
         task_input = {
@@ -86,7 +86,10 @@ class TestImportTask(test_utils.BaseTestCase):
         task_ttl = CONF.task.task_time_to_live
 
         self.task_type = 'import'
+        request_id = 'fake_request_id'
+        user_id = 'fake_user'
         self.task = self.task_factory.new_task(self.task_type, TENANT1,
+                                               UUID1, user_id, request_id,
                                                task_time_to_live=task_ttl,
                                                task_input=task_input)
 
@@ -120,11 +123,11 @@ class TestImportTask(test_utils.BaseTestCase):
         img_factory.new_image.side_effect = create_image
 
         with mock.patch.object(script_utils, 'get_image_data_iter') as dmock:
-            dmock.return_value = six.BytesIO(b"TEST_IMAGE")
+            dmock.return_value = io.BytesIO(b"TEST_IMAGE")
 
             with mock.patch.object(putils, 'trycmd') as tmock:
                 tmock.return_value = (json.dumps({
-                    'format': 'qcow2',
+                    'format': 'raw',
                 }), None)
 
                 executor.begin_processing(self.task.task_id)
@@ -163,7 +166,7 @@ class TestImportTask(test_utils.BaseTestCase):
         img_factory.new_image.side_effect = create_image
 
         with mock.patch.object(script_utils, 'get_image_data_iter') as dmock:
-            dmock.return_value = six.BytesIO(b"TEST_IMAGE")
+            dmock.return_value = io.BytesIO(b"TEST_IMAGE")
 
             with mock.patch.object(import_flow._ImportToFS, 'execute') as emk:
                 executor.begin_processing(self.task.task_id)
@@ -175,6 +178,39 @@ class TestImportTask(test_utils.BaseTestCase):
                 self.assertFalse(os.path.exists(tmp_image_path))
                 self.assertTrue(os.path.exists(image_path))
 
+    def test_import_flow_invalid_data_file(self):
+        self.config(engine_mode='serial',
+                    group='taskflow_executor')
+
+        img_factory = mock.MagicMock()
+
+        executor = taskflow_executor.TaskExecutor(
+            self.context,
+            self.task_repo,
+            self.img_repo,
+            img_factory)
+
+        self.task_repo.get.return_value = self.task
+
+        def create_image(*args, **kwargs):
+            kwargs['image_id'] = UUID1
+            return self.img_factory.new_image(*args, **kwargs)
+
+        self.img_repo.get.return_value = self.image
+        img_factory.new_image.side_effect = create_image
+
+        with mock.patch.object(script_utils, 'get_image_data_iter') as dmock:
+            dmock.return_value = io.BytesIO(b"TEST_IMAGE")
+
+            with mock.patch.object(putils, 'trycmd') as tmock:
+                out = json.dumps({'format-specific':
+                                  {'data': {'data-file': 'somefile'}}})
+                tmock.return_value = (out, '')
+                e = self.assertRaises(RuntimeError,
+                                      executor.begin_processing,
+                                      self.task.task_id)
+                self.assertIn('somefile', str(e))
+
     def test_import_flow_revert_import_to_fs(self):
         self.config(engine_mode='serial', group='taskflow_executor')
 
@@ -234,7 +270,7 @@ class TestImportTask(test_utils.BaseTestCase):
         img_factory.new_image.side_effect = create_image
 
         with mock.patch.object(script_utils, 'get_image_data_iter') as dmock:
-            dmock.return_value = six.BytesIO(b"TEST_IMAGE")
+            dmock.return_value = io.BytesIO(b"TEST_IMAGE")
 
             with mock.patch.object(putils, 'trycmd') as tmock:
                 tmock.return_value = (json.dumps({
@@ -284,7 +320,7 @@ class TestImportTask(test_utils.BaseTestCase):
         img_factory.new_image.side_effect = create_image
 
         with mock.patch.object(script_utils, 'get_image_data_iter') as dmock:
-            dmock.return_value = six.BytesIO(b"TEST_IMAGE")
+            dmock.return_value = io.BytesIO(b"TEST_IMAGE")
 
             with mock.patch.object(putils, 'trycmd') as tmock:
                 tmock.return_value = (json.dumps({
@@ -337,7 +373,7 @@ class TestImportTask(test_utils.BaseTestCase):
 
         with mock.patch.object(urllib.request, 'urlopen') as umock:
             content = b"TEST_IMAGE"
-            umock.return_value = six.BytesIO(content)
+            umock.return_value = io.BytesIO(content)
 
             with mock.patch.object(import_flow, "_get_import_flows") as imock:
                 imock.return_value = (x for x in [])
@@ -417,10 +453,10 @@ class TestImportTask(test_utils.BaseTestCase):
                                               self.task_type)
 
         data = [b"test"]
-
+        image_size = len(b"test")
         store = glance_store.get_store_from_scheme('file')
         path = glance_store.store_add_to_backend(mock.sentinel.image_id, data,
-                                                 mock.sentinel.image_size,
+                                                 image_size,
                                                  store, context=None)[0]
 
         path_wo_scheme = path.split("file://")[1]
